@@ -24,9 +24,6 @@
  *
  */
 
-#include <ui/interface.h>
-#include <ui/callbacks.h>
-#include <ui/sketchwidget.h>
 #include <gtk/gtk.h>
 #include <libintl.h>
 #include <sys/param.h>
@@ -42,6 +39,10 @@
 #include <hildon/hildon-font-selection-dialog.h>
 #include <hildon/hildon-color-button.h>
 
+#include "callbacks.h"
+#include "interface.h"
+#include "sketchwidget.h"
+
 /*
  * Privates: 
  */
@@ -97,6 +98,8 @@ MainView *interface_main_view_new(AppData * data)
         result->checklist_edited = FALSE;
         result->current_color = NULL;
         result->can_show_node_view = FALSE;
+        result->node_list_longpress_path = NULL;
+        result->checklist_longpress_source_id = 0;
 
 	GtkWidget *main_vbox = gtk_vbox_new(FALSE, 0);
 
@@ -135,6 +138,10 @@ MainView *interface_main_view_new(AppData * data)
 	create_toolbar(result);
 	GtkWidget *nodemenu = create_menu(result);
 
+        /* Handle the event when the longpress menu is shown */
+        g_signal_connect(G_OBJECT(nodemenu), "show",
+                G_CALLBACK(on_node_menu_show), result);
+
 	g_signal_connect_swapped(result->treeview, "button_press_event", G_CALLBACK(cb_popup), nodemenu);
 	gtk_widget_tap_and_hold_setup(GTK_WIDGET(result->treeview), nodemenu, NULL, GTK_TAP_AND_HOLD_NONE);
 
@@ -159,7 +166,8 @@ MainView *interface_main_view_new(AppData * data)
 	result->file_edited = FALSE;
 	result->file_name = NULL;
 
-	g_signal_connect(G_OBJECT(result->data->main_view), "key_press_event", G_CALLBACK(key_press_cb), result);
+        /* Signals on the main view */
+        g_signal_connect(G_OBJECT(result->data->main_view), "key-press-event", G_CALLBACK(on_main_view_key_press), result);
 	g_signal_connect(G_OBJECT(result->data->main_view), "delete_event", G_CALLBACK(close_cb), result);
 
 	sk_set_brushsize(result, 2);
@@ -296,20 +304,28 @@ GtkWidget *create_menu(MainView * main)
     /* Create the node window app menu */
     main->node_view_app_menu = GTK_WIDGET(hildon_app_menu_new());
 
-    main->menu_button_rename = hildon_button_new_with_text(0, 0, _("Rename"), NULL);
-    main->menu_button_delete = hildon_button_new_with_text(0, 0, _("Delete"), NULL);
+    main->menu_button_rename = hildon_button_new_with_text(0, 0, _("Rename memo"), NULL);
+    main->menu_button_delete = hildon_button_new_with_text(0, 0, _("Delete memo"), NULL);
     main->menu_button_export = hildon_button_new_with_text(0, 0, _("Export to file"), NULL);
-    main->menu_button_clear = hildon_button_new_with_text(0, 0, _("Clear"), NULL);
+    main->menu_button_clear = hildon_button_new_with_text(0, 0, _("Clear contents"), NULL);
+    main->menu_button_square = hildon_check_button_new(0);
+    gtk_button_set_label(GTK_BUTTON(main->menu_button_square), _("Square shapes"));
+    main->menu_button_filled = hildon_check_button_new(0);
+    gtk_button_set_label(GTK_BUTTON(main->menu_button_filled), _("Fill shapes"));
 
     g_signal_connect(G_OBJECT(main->menu_button_rename), "clicked", G_CALLBACK(callback_file_rename_node), main);
     g_signal_connect(G_OBJECT(main->menu_button_delete), "clicked", G_CALLBACK(callback_file_delete_node), main);
     g_signal_connect(G_OBJECT(main->menu_button_export), "clicked", G_CALLBACK(callback_file_export_node), main);
     g_signal_connect(G_OBJECT(main->menu_button_clear), "clicked", G_CALLBACK(callback_edit_clear), main);
+    g_signal_connect(G_OBJECT(main->menu_button_square), "toggled", G_CALLBACK(callback_sketch_button_toggled), main);
+    g_signal_connect(G_OBJECT(main->menu_button_filled), "toggled", G_CALLBACK(callback_sketch_button_toggled), main);
 
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_rename));
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_delete));
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_export));
     hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_clear));
+    hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_square));
+    hildon_app_menu_append(HILDON_APP_MENU(main->node_view_app_menu), GTK_BUTTON(main->menu_button_filled));
 
     hildon_window_set_app_menu(HILDON_WINDOW(main->data->node_view),
             HILDON_APP_MENU(main->node_view_app_menu));
@@ -513,31 +529,38 @@ GtkWidget *create_sketchlinesmenu(MainView * main)
 
 GtkWidget *create_shapemenu(MainView * main)
 {
-	GtkWidget *menu;
-	int i;
-
+	GtkWidget* menu = NULL;
 	GSList *gr = NULL;
+	int i = 0;
+
+        const gchar* labels[] = {
+            _("Freehand"),
+            _("Line"),
+            _("Rectangle"),
+            _("Ellipse"),
+        };
 
 	menu = hildon_gtk_menu_new();
 
-	char st[4][128];
+	for(i=0; i<SKETCHSHAPE_COUNT; i++) {
+            gchar* filename = g_strdup_printf("%s/shape%d.png", PIXMAPDIR, i);
+            GtkWidget* pix = gtk_image_new_from_file(filename);
+            if (pix == NULL) {
+                g_error("Cannot load file: %s", filename);
+            }
+            g_object_ref(pix);
+            g_free(filename);
 
-	g_snprintf(st[0], sizeof(st[0]), _("Free"));
-	g_snprintf(st[1], sizeof(st[1]), _("Line"));
-	g_snprintf(st[2], sizeof(st[2]), _("Rectangle"));
-	g_snprintf(st[3], sizeof(st[3]), _("Ellipse"));
+            main->shapemenuitems[i] = gtk_radio_menu_item_new_with_label(gr, labels[i]);
+            gr = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(main->shapemenuitems[i]));
 
-	for(i = 0; i < 4; i++)
-		{
-			main->shapemenuitems[i] = gtk_radio_menu_item_new_with_label(gr, st[i]);
-			gr = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(main->shapemenuitems[i]));
+            gtk_object_set_user_data(GTK_OBJECT(main->shapemenuitems[i]), GINT_TO_POINTER(i));
+            gtk_object_set_data(GTK_OBJECT(main->shapemenuitems[i]), "m", main);
+            gtk_object_set_data(GTK_OBJECT(main->shapemenuitems[i]), "i", pix);
 
-			gtk_object_set_user_data(GTK_OBJECT(main->shapemenuitems[i]), GINT_TO_POINTER(i));
-			gtk_object_set_data(GTK_OBJECT(main->shapemenuitems[i]), "m", main);
-
-			gtk_menu_prepend(menu, main->shapemenuitems[i]);
-			g_signal_connect(G_OBJECT(main->shapemenuitems[i]), "activate", G_CALLBACK(callback_shapemenu), main->shapemenuitems[i]);
-		}
+            gtk_menu_prepend(menu, main->shapemenuitems[i]);
+            g_signal_connect(G_OBJECT(main->shapemenuitems[i]), "activate", G_CALLBACK(callback_shapemenu), main->shapemenuitems[i]);
+        }
 
 	gtk_widget_show_all(GTK_WIDGET(menu));
 
@@ -639,7 +662,6 @@ static void create_toolbar(MainView * main)
 	g_signal_connect(G_OBJECT(main->eraser_tb), "toggled", G_CALLBACK(callback_eraser), main);
 
 	main->shape_tb = gtk_tool_button_new(NULL, NULL);
-	_tool_button_set_icon_file(GTK_TOOL_BUTTON(main->shape_tb), PIXMAPDIR "/shapes.png");
 	gtk_toolbar_insert(GTK_TOOLBAR(main->toolbar), main->shape_tb, -1);
 	g_signal_connect(G_OBJECT(main->shape_tb), "clicked", G_CALLBACK(callback_menu), main->shapemenu);
 
@@ -851,6 +873,10 @@ void create_treeview(MainView * main)
 void create_checklist(MainView * main)
 {
 	main->listscroll = hildon_pannable_area_new();
+
+        /* Detect scrolling to abort longpress action on checklist */
+        g_signal_connect(G_OBJECT(main->listscroll), "panning-started", G_CALLBACK(on_checklist_start_panning), main);
+
 	gtk_widget_hide(main->listscroll);
 
 	GtkListStore *store = gtk_list_store_new(CHECKN_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_STRING);
@@ -864,14 +890,12 @@ void create_checklist(MainView * main)
 
 	GtkCellRenderer *renderer2 = gtk_cell_renderer_text_new();
 
-	GtkTreeViewColumn *col1 = gtk_tree_view_column_new_with_attributes("", renderer1, 
-/*                                                      "cell-background", CHECKNODE_COLOR,*/
+	main->checklist_column_check = gtk_tree_view_column_new_with_attributes("", renderer1,
                                                       "icon-name", CHECKNODE_ICON_NAME,
                                                       NULL);
 
-	main->listtextcol = gtk_tree_view_column_new_with_attributes("", renderer2, 
+	main->checklist_column_text = gtk_tree_view_column_new_with_attributes("", renderer2,
                                                       "text", CHECKNODE_TEXT,
-/*                                                      "cell-background", CHECKNODE_COLOR,*/
                                                       "foreground", CHECKNODE_COLOR,
                                                       "strikethrough", CHECKNODE_STRIKE,
                                                       "weight", CHECKNODE_BOLD,
@@ -880,13 +904,13 @@ void create_checklist(MainView * main)
         /* Ellipsize to avoid horizontal scroll bars */
         g_object_set(G_OBJECT(renderer2), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 
-	gtk_tree_view_append_column(GTK_TREE_VIEW(main->listview), col1);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(main->listview), main->listtextcol);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(main->listview), main->checklist_column_check);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(main->listview), main->checklist_column_text);
 
         /* Configure for speedier scrolling - fixed height mode */
-	gtk_tree_view_column_set_sizing(col1, GTK_TREE_VIEW_COLUMN_FIXED);
-        gtk_tree_view_column_set_fixed_width(col1, 40);
-	gtk_tree_view_column_set_sizing(main->listtextcol, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_sizing(main->checklist_column_check, GTK_TREE_VIEW_COLUMN_FIXED);
+        gtk_tree_view_column_set_fixed_width(main->checklist_column_check, 40);
+	gtk_tree_view_column_set_sizing(main->checklist_column_text, GTK_TREE_VIEW_COLUMN_FIXED);
         gtk_tree_view_set_fixed_height_mode(GTK_TREE_VIEW(main->listview), TRUE);
 
 	GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(main->listview));
@@ -903,6 +927,10 @@ void create_checklist(MainView * main)
 
 	gtk_object_set_user_data(GTK_OBJECT(main->listview), (gpointer) main);
 
+        /* Add handlers for checklist interaction */
+        g_signal_connect(main->listview, "button-press-event", G_CALLBACK(on_checklist_button_press), main);
+        g_signal_connect(main->listview, "button-release-event", G_CALLBACK(on_checklist_button_release), main);
+
 	gtk_container_add(GTK_CONTAINER(main->listscroll), main->listview);
 	gtk_widget_show(main->listview);
 /*
